package com.mortals.xhx.module.ph.web;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.util.ThreadPool;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.GroupListEnum;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.site.SitePdu;
import com.mortals.xhx.feign.site.ISiteFeign;
import com.mortals.xhx.module.ph.model.PhQueueQuery;
import com.mortals.xhx.module.ph.model.PhQueueStatEntity;
import com.mortals.xhx.module.ph.model.PhQueueStatQuery;
import com.mortals.xhx.module.ph.service.PhQueueStatService;
import com.mortals.xhx.module.pj.model.PjEvaluateStatEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 排队叫号汇总统计
 *
 * @author zxfei
 * @date 2024-06-24
 */
@RestController
@RequestMapping("ph/queue/stat")
@Slf4j
public class PhQueueStatController extends BaseCRUDJsonBodyMappingController<PhQueueStatService, PhQueueStatEntity, Long> {

    @Autowired
    private ParamService paramService;

    @Autowired
    private ISiteFeign siteFeign;
    @Autowired
    private PhQueueStatService phQueueStatService;

    public PhQueueStatController() {
        super.setModuleDesc("排队叫号汇总统计");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {

        //通过站点 查询存在的部门 大厅等
        Object siteId = model.get("siteId");
        if (!ObjectUtils.isEmpty(siteId)) {
            PhQueueStatQuery queueStatQuery = new PhQueueStatQuery();
            queueStatQuery.setSiteId((Long) siteId);
            List<PhQueueStatEntity> phQueueStatEntities = this.service.find(queueStatQuery, context);
            Map<String, String> sectionNameMap = phQueueStatEntities.stream().filter(f -> !ObjectUtils.isEmpty(f.getSectionName())).collect(Collectors.toMap(PhQueueStatEntity::getSectionName, PhQueueStatEntity::getSectionName, (k1, k2) -> k2));
            Map<String, String> hallNameMap = phQueueStatEntities.stream().filter(f -> !ObjectUtils.isEmpty(f.getHallName())).collect(Collectors.toMap(PhQueueStatEntity::getHallName, PhQueueStatEntity::getHallName, (k1, k2) -> k2));
            Map<String, String> windowFromnumMap = phQueueStatEntities.stream().filter(f -> !ObjectUtils.isEmpty(f.getWindowFromnum())).collect(Collectors.toMap(PhQueueStatEntity::getWindowFromnum, PhQueueStatEntity::getWindowFromnum, (k1, k2) -> k2));
            Map<String, String> bussinessMap = phQueueStatEntities.stream().filter(f -> !ObjectUtils.isEmpty(f.getBusiness())).collect(Collectors.toMap(PhQueueStatEntity::getBusiness, PhQueueStatEntity::getBusiness, (k1, k2) -> k2));

            this.addDict(model, "sectionNameList", sectionNameMap);
            this.addDict(model, "hallNameList", hallNameMap);
            this.addDict(model, "windowFromnumList", windowFromnumMap);
            this.addDict(model, "businessList", bussinessMap);
            this.addDict(model, "groupList", GroupListEnum.getEnumMap());
        }

        super.init(model, context);
    }


    @Override
    protected void doListBefore(PhQueueStatEntity query, Map<String, Object> model, Context context) throws AppException {
        if (ObjectUtils.isEmpty(query.getHallName())) query.setHallName("");
        if (ObjectUtils.isEmpty(query.getSectionName())) query.setSectionName("");
        if (ObjectUtils.isEmpty(query.getBusiness())) query.setBusiness("");
        if (ObjectUtils.isEmpty(query.getWindowFromnum())) query.setWindowFromnum("");
        addGroup(query);
        model.put("siteId", query.getSiteId());
    }

    private static void addGroup(PhQueueStatEntity query) {
        if (!ObjectUtils.isEmpty(query.getSiteId())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("siteId")) {
                groupList.add("siteId");
            }
        }

        if (!ObjectUtils.isEmpty(query.getSectionName())
                ||!ObjectUtils.isEmpty(query.getSectionNameList())
                ||!ObjectUtils.isEmpty(query.getSectionNameNotList())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("sectionName")) {
                groupList.add("sectionName");
            }
        }
        if (!ObjectUtils.isEmpty(query.getHallName())
                ||!ObjectUtils.isEmpty(query.getHallNameList())
                ||!ObjectUtils.isEmpty(query.getHallNameNotList())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("hallName")) {
                groupList.add("hallName");
            }
        }

        if (!ObjectUtils.isEmpty(query.getWindowFromnum())
                ||!ObjectUtils.isEmpty(query.getWindowFromnumList())
                ||!ObjectUtils.isEmpty(query.getWindowFromnumNotList())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("windowFromnum")) {
                groupList.add("windowFromnum");
            }
        }

        if (!ObjectUtils.isEmpty(query.getBusiness())
                ||!ObjectUtils.isEmpty(query.getBusinessList())
                ||!ObjectUtils.isEmpty(query.getHallNameNotList())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("business")) {
                groupList.add("business");
            }
        }

    }

    @PostMapping(value = "/stat")
    @UnAuth
    public Rest<String> addAttendanceRecordCustom(@RequestBody PhQueueQuery phQueueQuery) {
        Rest<String> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code = 1;
        try {

            Runnable runnable = new Runnable() {
                @Override
                public void run() {
                    //天数区间分段计算
                    DateTime attendStart = DateUtil.parseDate(phQueueQuery.getTaketimeStart());
                    DateTime attendEnd = DateUtil.parseDate(phQueueQuery.getTaketimeEnd());
                    Long compare = DateUtil.between(attendEnd, attendStart, DateUnit.DAY);
                    StopWatch stopWatch = new StopWatch("stopwatch");
                    log.info("计算天数区间：{}", compare);
                    SitePdu sitePdu = new SitePdu();
                    sitePdu.setSize(-1);
                    Rest<RespData<List<SitePdu>>> resp = siteFeign.list(sitePdu);
                    for (int i = 0; i <= compare.intValue(); i++) {
                        DateTime curDate = DateUtil.offsetDay(attendStart, i);
                        log.info("记录日期：{}", curDate.toDateStr());
                        stopWatch.start("执行本地方法");

                        if (resp.getCode() == 1) {
                            List<SitePdu> sitePduList = resp.getData().getData();
                            sitePduList.stream().forEach(site -> {
                                PhQueueStatEntity sitestatEntity = new PhQueueStatEntity();
                                sitestatEntity.initAttrValue();
                                sitestatEntity.setSiteId(site.getId());
                                sitestatEntity.setSiteName(site.getSiteName());
                                sitestatEntity.setSiteCode(site.getSiteCode());

                                sitestatEntity.setYear(curDate.year());
                                sitestatEntity.setMonth(curDate.month() + 1);
                                sitestatEntity.setDay(curDate.dayOfMonth());
                                //设置年月日
                                phQueueStatService.updateSitePhStat(sitestatEntity, context);
                            });
                        }
                        stopWatch.stop();
                        log.info("日期：{} 完成,耗时：{}ms", curDate.toDateStr(), stopWatch.getLastTaskTimeMillis());
                    }
                }
            };

            ThreadPool.getInstance().execute(runnable);


            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = -1;
            this.doException(this.request, busiDesc, model, e);
            model.put("message_info", e.getMessage());
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;

    }
}